//-----------------------------------------------
// BowSound
// 2002/11/28  Kazuto Sato
//-----------------------------------------------
#include <OPENR/OPENRAPI.h>
#include <OPENR/OSyslog.h>
#include <OPENR/core_macro.h>
#include "BowSound.h"

BowSound::BowSound() : soundState(SS_IDLE),
                                   speakerID(oprimitiveID_UNDEF),
                                   soundID(odesigndataID_UNDEF),
                                   wav()
{
    for (int i = 0; i < SOUND_NUM_BUFFER; i++) region[i] = 0;
}

OStatus
BowSound::DoInit(const OSystemEvent& event)
{
    OSYSDEBUG(("BowSound::DoInit()\n"));

    NEW_ALL_SUBJECT_AND_OBSERVER;
    REGISTER_ALL_ENTRY;
    SET_ALL_READY_AND_NOTIFY_ENTRY;

    OpenSpeaker();
    NewSoundVectorData();
    LoadWAV();
    SetPowerAndVolume();

    return oSUCCESS;
}

OStatus
BowSound::DoStart(const OSystemEvent& event)
{
    OSYSDEBUG(("BowSound::DoStart()\n"));

    soundState = SS_START;

    ENABLE_ALL_SUBJECT;
    ASSERT_READY_TO_ALL_OBSERVER;

    return oSUCCESS;
}

OStatus
BowSound::DoStop(const OSystemEvent& event)
{
    OSYSDEBUG(("BowSound::DoStop()\n"));

    soundState = SS_IDLE;

    DISABLE_ALL_SUBJECT;
    DEASSERT_READY_TO_ALL_OBSERVER;

    return oSUCCESS;
}

OStatus
BowSound::DoDestroy(const OSystemEvent& event)
{
    OPENR::DeleteDesignData(soundID);
    DELETE_ALL_SUBJECT_AND_OBSERVER;
    return oSUCCESS;
}

void
BowSound::NotifyCommand(const ONotifyEvent& event)
{
    OSYSDEBUG(("BowSound::NotifyCommand()\n"));
    
    // AiboBiffؤηϾά
    if (soundState == SS_START) {
        Play();
        soundState = SS_PLAYING;
    }
    
    observer[event.ObsIndex()]->AssertReady();
}

void
BowSound::ReadyPlay(const OReadyEvent& event)
{
    OSYSDEBUG(("BowSound::ReadyPlay()\n"));

    if (soundState == SS_PLAYING) {

        RCRegion* rgn = FindFreeRegion();
        if (CopyWAVTo(rgn) == WAV_SUCCESS) {
            subject[sbjPlay]->SetData(rgn);
            subject[sbjPlay]->NotifyObservers();
        } else {
            if (IsAllRegionFree() == true) {
                // AiboBiffؤηϾά
                soundState = SS_START;
            }
        }

    }
}

void
BowSound::Play()
{
    OSYSDEBUG(("BowSound::Play()\n"));

    wav.Rewind();

    for (int i = 0; i < SOUND_NUM_BUFFER; i++) {
        if (CopyWAVTo(region[i]) == WAV_SUCCESS)
            subject[sbjPlay]->SetData(region[i]);
    }
    subject[sbjPlay]->NotifyObservers();
}

void
BowSound::OpenSpeaker()
{
    OStatus result = OPENR::OpenPrimitive(SPEAKER_LOCATOR, &speakerID);
    if (result != oSUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "BowSound::OpenSpeaker()",
                  "OPENR::OpenPrimitive() FAILED", result));
    }
}

void
BowSound::NewSoundVectorData()
{
    OStatus result;
    MemoryRegionID    soundVecDataID;
    OSoundVectorData* soundVecData;

    for (int i = 0; i < SOUND_NUM_BUFFER; i++) {

        result = OPENR::NewSoundVectorData(1, SOUND_UNIT_SIZE,
                                           &soundVecDataID, &soundVecData);
        if (result != oSUCCESS) {
            OSYSLOG1((osyslogERROR, "%s : %s %d",
                      "BowSound::NewSoundVectorData()",
                      "OPENR::NewSoundVectorData() FAILED", result));
            return;
        }

        soundVecData->SetNumData(1);
        soundVecData->GetInfo(0)->Set(odataSOUND_VECTOR,
                                      speakerID, SOUND_UNIT_SIZE);

        region[i] = new RCRegion(soundVecData->vectorInfo.memRegionID,
                                 soundVecData->vectorInfo.offset,
                                 (void*)soundVecData,
                                 soundVecData->vectorInfo.totalSize);
    }
}

void
BowSound::LoadWAV()
{
    OStatus result;
    size_t size;
    byte*  addr;

    result = OPENR::FindDesignData(BOW_SOUND,
                                   &soundID, &addr, &size);
    if (result != oSUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "BowSound::LoadWAV()",
                  "OPENR::FindDesignData() FAILED", result));
        return;
    }

    wav.Set(addr);
}

void
BowSound::SetPowerAndVolume()
{
    OStatus result;

    result = OPENR::ControlPrimitive(speakerID,
                                     oprmreqSPEAKER_MUTE_ON, 0, 0, 0, 0);
    if (result != oSUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "BowSound::SetPowerAndVolume()", 
                  "OPENR::ControlPrimitive(oprmreqSPEAKER_MUTE_ON) FAILED",
                  result));
    }

    result = OPENR::SetMotorPower(opowerON);
    if (result != oSUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "BowSound::SetPowerAndVolume()", 
                  "OPENR::SetMotorPower() FAILED", result));
    }

    result = OPENR::ControlPrimitive(speakerID,
                                     oprmreqSPEAKER_MUTE_OFF, 0, 0, 0, 0);
    if (result != oSUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "BowSound::SetPowerAndVolume()", 
                  "OPENR::ControlPrimitive(oprmreqSPEAKER_MUTE_OFF) FAILED",
                  result));
    }

    OPrimitiveControl_SpeakerVolume volume(ospkvol10dB);
    result = OPENR::ControlPrimitive(speakerID,
                                     oprmreqSPEAKER_SET_VOLUME,
                                     &volume, sizeof(volume), 0, 0);
    if (result != oSUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "BowSound::SetPowerAndVolume()", 
                  "OPENR::ControlPrimitive(oprmreqSPEAKER_SET_VOLUME) FAILED",
                  result));
    }
}

WAVError
BowSound::CopyWAVTo(RCRegion* rgn)
{
    OSoundVectorData* soundVecData = (OSoundVectorData*)rgn->Base();
    return wav.CopyTo(soundVecData);
}

RCRegion*
BowSound::FindFreeRegion()
{
    for (int i = 0; i < SOUND_NUM_BUFFER; i++) {
        if (region[i]->NumberOfReference() == 1) return region[i];
    }

    return 0;
}

bool
BowSound::IsAllRegionFree()
{
    for (int i = 0; i < SOUND_NUM_BUFFER; i++) {
        if (region[i]->NumberOfReference() > 1) return false;
    }

    return true;
}
